# -*- coding: utf-8 -*-
from django.shortcuts import get_object_or_404, render, redirect
from django.http import HttpResponse, Http404
from django.forms.utils import ErrorList
from django.forms.widgets import HiddenInput
from django.contrib.auth.models import User

from lib.access_auth import permission_needed
from lib.tools import erreur
from .models import (
    Inscription,
    AffichageChampCaseACocher,
    AffichageChamp,
    MODELES_CHAMP,
    DonneeFormulaire,
    enregistrer_photos,
    invalidate_noms_prenoms,
    lire_photo,
    LdapCriError,
    EtatInscr,
    preparer_inscription,
    donnees_cri,
    a_photo_enreg,
    selectionner_photos,
    get_filieres,
    liste_photos,
    lire_photo_enreg,
    GENRES,
)
from .creation_comptes import creer_compte, prochain_uid_libre
from inscriptions.views import valider_formulaire
from ldap_inp.utf_tools import to_alphabetic
from datetime import datetime, date, timedelta
from .forms import choix_promo, InscriptionForm, TypeChampForm, ModelFormGenerique
from django.utils.html import escape
from django.utils.safestring import mark_safe
from portailuser.models import Ecole

import urllib
import csv

genre = [(str(sexe[0]), sexe[1]) for sexe in GENRES]


@permission_needed("gerer_inscriptions")
def liste(request):
    nettoyage = request.session.get("nettoyage", False)
    if nettoyage:
        del request.session["nettoyage"]

    redir_inscr = request.session.get("redir_inscr", True)

    if request.method == "POST":
        request.session["redir_inscr"] = request.POST.get("redir_inscr", "1") == "1"
        return redirect("gestion:inscriptions:liste")

    c = {
        "inscriptions": Inscription.objects.filter(ecole=request.ecole, etat=0).order_by("fiche_imp", "nom"),
        "redir_inscr": redir_inscr,
        "nettoyage": nettoyage,
    }
    return render(request, "gestion/inscriptions/liste.html", c)


@permission_needed("gerer_inscriptions")
def details(request):
    champs = [
        AffichageChamp("login", "Login"),
        AffichageChamp("filiere", "Filière", [(None, "Toutes")] + get_filieres(request.ecole)),
        AffichageChamp("promo", "Promo", [(None, "Toutes")] + choix_promo),
        AffichageChamp("genre", "Genre", [(None, "Tous")] + genre),
        AffichageChampCaseACocher("inscrit_ae", "Inscrit à l’AE"),
        AffichageChampCaseACocher("inscrit_frappe", "Inscrit sur la frappe"),
    ]

    nb_colonnes = 1
    entete = ""

    for type_champ in MODELES_CHAMP.values():
        for champ in type_champ.objects.filter(ecole=request.ecole):
            champs.append(champ.affichage_champ())

    for champ in champs:
        champ.recup_parametres(request.GET)
        if champ.est_affiche:
            if champ.nom:
                entete += f"<th>{escape(champ.nom)}</th>"
            nb_colonnes += 1

    inscriptions_filtrees = []
    for inscription in Inscription.objects.filter(ecole=request.ecole).order_by("nom", "prenom"):
        inscription.charger_attrs_persos()
        colonnes = ""
        a_afficher = True

        for champ in champs:
            if not champ.filtrer(inscription):
                a_afficher = False
                break

            if champ.est_affiche:
                colonnes += "<td>{}</td>".format(escape(champ.afficher(inscription)).replace("\n", "<br />\n"))

        if not a_afficher:
            continue

        inscription.colonnes = mark_safe(colonnes)
        inscriptions_filtrees.append(inscription)

    c = {
        "champs": champs,
        "nb_colonnes": nb_colonnes,
        "entete": mark_safe(entete),
        "nb_entrees": len(inscriptions_filtrees),
        "entrees": inscriptions_filtrees,
        "parametres": request.GET.urlencode(),
    }

    return render(request, "gestion/inscriptions/details.html", c)


@permission_needed("gerer_inscriptions")
def nettoyage(request):
    if request.method == "POST":
        if "conf_supprimer" in request.POST:
            Inscription.objects.filter(
                ecole=request.ecole,
                etat=3,
                date_ajout__lt=datetime.today() - timedelta(weeks=4),
            ).delete()
            request.session["nettoyage"] = True

            return redirect("gestion:inscriptions:liste")

        elif "conf_annuler" in request.POST:
            return redirect("gestion:inscriptions:details")

    c = {
        "liste_inscr": Inscription.objects.filter(
            ecole=request.ecole,
            etat=3,
            date_ajout__lt=datetime.today() - timedelta(weeks=4),
        )
    }

    return render(request, "gestion/inscriptions/nettoyage.html", c)


def vers_utf8(liste):
    return [i if isinstance(i, str) else i for i in liste]


@permission_needed("gerer_inscriptions")
def details_csv(request):
    champs = [
        AffichageChamp("login", "Login"),
        AffichageChamp("filiere", "Filière", [(None, "Toutes")] + get_filieres(request.ecole)),
        AffichageChamp("promo", "Promo", [(None, "Toutes")] + choix_promo),
        AffichageChamp("genre", "Genre", [(None, "Tous")] + genre),
        AffichageChampCaseACocher("inscrit_ae", "Inscrit à l’AE"),
        AffichageChampCaseACocher("inscrit_frappe", "Inscrit sur la frappe"),
    ]

    entete = ["Nom"]
    reponse = HttpResponse(content_type="text/csv")
    reponse["Content-Disposition"] = "attachment;filename=details.csv"
    csv_writer = csv.writer(reponse, dialect="excel")

    for type_champ in MODELES_CHAMP.values():
        for champ in type_champ.objects.filter(ecole=request.ecole):
            champs.append(champ.affichage_champ())

    for champ in champs:
        champ.recup_parametres(request.GET)
        if champ.est_affiche:
            if champ.nom:
                entete.append(champ.nom)

    csv_writer.writerow(vers_utf8(entete))

    for inscription in Inscription.objects.filter(ecole=request.ecole).order_by("nom", "prenom"):
        inscription.charger_attrs_persos()
        colonnes = [inscription.prenom + " " + inscription.nom]
        a_afficher = True

        for champ in champs:
            if not champ.filtrer(inscription):
                a_afficher = False
                break

            if champ.est_affiche:
                colonnes.append(champ.afficher(inscription))

        if not a_afficher:
            continue

        csv_writer.writerow(vers_utf8(colonnes))

    return reponse


def photos_supplementaires(donnees):
    liste = []
    for i in donnees:
        if i[0:14] == "photo_supplem_":
            liste.append(i[14:])
    return liste


def afficher_page_nouvelle_inscription_admin(
    request, id=None, form: InscriptionForm = None, photos_supplem=[], photo_choisie=None, err_photo=False
):
    if id is not None:
        inscription = get_object_or_404(Inscription, id=id)
    else:
        inscription = None

    # Date initiale dans le champ de sélection de date
    initial = {
        "date_naissance": inscription.date_naissance
        if inscription
        else date(year=date.today().year - 20, month=1, day=1),
    }

    inscr_nom = request.session.get("inscr_nom", None)
    inscr_edit = request.session.get("inscr_edit", False)
    if inscr_nom:
        del request.session["inscr_nom"]
        del request.session["inscr_edit"]

    filiere_init = request.session.get("filiere_init", None)
    err_photo = False

    if form is None:
        form = InscriptionForm(request.ecole, filiere_init, instance=inscription, initial=initial)

    c = {
        "inscription": inscription,
        "form": form,
        "inscr_nom": inscr_nom,
        "inscr_edit": inscr_edit,
        "photos": [(urllib.parse.quote(i), False, False) for i in liste_photos()],
        "err_photo": err_photo,
    }

    return render(request, "gestion/inscriptions/edition.html", c)


@permission_needed("gerer_inscriptions")
def edition(request, id=None):
    if id is not None:
        inscription = get_object_or_404(Inscription, id=id)
    else:
        inscription = None

    # Date initiale dans le champ de sélection de date
    initial = {
        "date_naissance": inscription.date_naissance
        if inscription
        else date(year=date.today().year - 20, month=1, day=1),
    }

    if request.method != "POST":
        return afficher_page_nouvelle_inscription_admin(request, id)

    photos_supplem = photos_supplementaires(request.POST.keys())
    photo_choisie = request.POST.get("photo_choisie", None)

    filiere_init = request.session.get("filiere_init", None)

    form = InscriptionForm(
        request.ecole,
        filiere_init,
        request.POST,
        instance=inscription,
        initial=initial,
    )
    if not form.is_valid():
        return afficher_page_nouvelle_inscription_admin(request, id, form, photos_supplem, photo_choisie)

    request.session["filiere_init"] = form.cleaned_data["filiere"]

    login = valider_formulaire(form)
    if login is None:
        return afficher_page_nouvelle_inscription_admin(request, id, form, photos_supplem, photo_choisie)

    inscr = form.save(commit=False)

    inscr.login = login
    inscr.ecole = request.ecole
    inscr.mdp = User.objects.make_random_password(length=20)

    donnees_photos = selectionner_photos(photo_choisie, photos_supplem)
    if donnees_photos is None:
        return afficher_page_nouvelle_inscription_admin(
            request, id, form, photos_supplem, photo_choisie, err_photo=True
        )

    inscr.save()

    enregistrer_photos(donnees_photos, inscr.id)

    for type_champ in MODELES_CHAMP.values():
        for champ in type_champ.objects.filter(ecole=request.ecole):
            try:
                donnee = DonneeFormulaire.objects.get(inscription=inscr, ident_champ=champ.ident)
            except DonneeFormulaire.DoesNotExist:
                donnee = DonneeFormulaire(inscription=inscr, ident_champ=champ.ident)
            donnee.valeur = None if form.cleaned_data[champ.ident] is None else str(form.cleaned_data[champ.ident])
            donnee.save()

    if request.session.get("redir_inscr", True):
        request.session["inscr_nom"] = inscr.prenom + " " + inscr.nom
        request.session["inscr_edit"] = False if id is None else True
        return redirect("gestion:inscriptions:edition")

    return redirect("gestion:inscriptions:fiche", id=inscr.id)


@permission_needed("gerer_inscriptions")
def suppression(request, id):
    inscription = get_object_or_404(Inscription, id=id)

    if request.method == "POST":
        if "conf_supprimer" in request.POST:
            inscription.delete()

            invalidate_noms_prenoms()

        return redirect("gestion:inscriptions:liste")

    c = {
        "inscription": inscription,
    }

    return render(request, "gestion/inscriptions/suppression.html", c)


@permission_needed("gerer_inscriptions")
def fiche(request, id):
    inscription = get_object_or_404(Inscription, id=id)
    inscription.fiche_imp = True
    inscription.save()

    shortFiliere = inscription.aff_filiere()

    c = {
        "inscr": inscription,
        "shortFiliere": shortFiliere,
        "a_photo": a_photo_enreg(id),
    }

    return render(request, "gestion/inscriptions/fiche.html", c)


@permission_needed("gerer_inscriptions")
def liste_champs(request):
    champs = []

    for type_champ in MODELES_CHAMP.values():
        champs += type_champ.objects.filter(ecole=request.ecole)

    c = {
        "champs": champs,
    }

    return render(request, "gestion/inscriptions/liste_champs.html", c)


@permission_needed("gerer_inscriptions")
def nouv_champ(request):
    if request.method == "POST":
        form = TypeChampForm(request.POST)
        if form.is_valid():
            return redirect(
                "gestion:inscriptions:nouv_champ_type",
                type=form.cleaned_data["type_champ"],
            )
    else:
        form = TypeChampForm()

    c = {
        "form": form,
    }

    return render(request, "gestion/inscriptions/nouv_champ.html", c)


@permission_needed("gerer_inscriptions")
def nouv_champ_type(request, type, id=None):
    try:
        type_champ = MODELES_CHAMP[type]
    except KeyError:
        raise Http404

    if id is not None:
        champ = get_object_or_404(type_champ, id=id)
        id = 1  # évite le 0 pour le template
    else:
        champ = None

    if request.method == "POST":
        form = ModelFormGenerique(type_champ, request.POST, instance=champ)

        # assure que le formulaire ait le bon type
        if champ:
            form.fields["ident"].widget = HiddenInput()

        if form.is_valid():
            champ_prec = champ  # pour être sûr que l'ident ne soit pas modifié
            champ = form.save(commit=False)

            unique = True

            if champ_prec:
                champ.ident = champ_prec.ident
            else:
                for type_champ in MODELES_CHAMP.values():
                    if type_champ.objects.filter(ecole=request.ecole, ident=champ.ident).count() > 0:
                        unique = False

            erreurs = champ.erreurs_champs()

            if unique and not erreurs:
                champ.ecole = request.ecole
                champ.save()
                return redirect("gestion:inscriptions:liste_champs")

            if not unique:
                form.errors["ident"] = ErrorList(["Identifiant non unique."])

            if erreurs:
                form.errors.update(erreurs)

    else:
        form = ModelFormGenerique(type_champ, instance=champ)

        if champ:
            form.fields["ident"].widget = HiddenInput()

    c = {
        "type": type_champ.nom_type,
        "form": form,
        "id": id,
    }

    return render(request, "gestion/inscriptions/nouv_champ_type.html", c)


@permission_needed("gerer_inscriptions")
def suppression_champ(request, type, id):
    try:
        type_champ = MODELES_CHAMP[type]
    except KeyError:
        raise Http404

    champ = get_object_or_404(type_champ, id=id)

    if request.method == "POST":
        if "conf_supprimer" in request.POST:
            champ.delete()

        return redirect("gestion:inscriptions:liste_champs")

    c = {
        "champ": champ,
    }

    return render(request, "gestion/inscriptions/suppression_champ.html", c)


@permission_needed("gerer_inscriptions")
def photo_src(request, nom):
    donnees = lire_photo(nom)
    if not donnees:
        raise Http404

    reponse = HttpResponse(donnees, content_type="image/jpeg")

    return reponse


@permission_needed("gerer_inscriptions")
def photo_dst(request, id):
    reponse = HttpResponse(lire_photo_enreg(id), content_type="image/jpeg")

    return reponse


@permission_needed("gerer_inscriptions")
def preparation(request):
    request.desactiver_widgets = True

    inscriptions = Inscription.objects.filter(ecole=request.ecole, etat__in=(1, 2)).order_by("nom", "prenom")
    inscriptions.update(etat=1)

    for inscription in inscriptions:
        try:
            (code, message) = preparer_inscription(inscription, avec_cri=request.ecole.ldap_cri)
        except LdapCriError:
            return erreur(
                request,
                "Le LDAP du CRI a rencontré un erreur. " "Veuillez crier un bon coup, et recharger cette page.",
                statut=500,
            )
        inscription.code = code
        inscription.message = mark_safe(message) if message else message
        if code != EtatInscr.ERREUR:
            inscription.etat = 2
        inscription.save()

    c = {
        "inscriptions": inscriptions,
    }
    return render(request, "gestion/inscriptions/preparation.html", c)


@permission_needed("gerer_inscriptions")
def effectuer(request):
    inscriptions = Inscription.objects.filter(ecole=request.ecole, etat__in=(1, 2))
    inscriptions.update(etat=1)

    for inscription in inscriptions:
        try:
            (code, message) = preparer_inscription(inscription, avec_cri=request.ecole.ldap_cri)
        except LdapCriError:
            return erreur(
                request,
                "Le LDAP du CRI a rencontré un erreur. " "Veuillez crier un bon coup, et recharger cette page.",
                statut=500,
            )
        inscription.code = code
        inscription.message = mark_safe(message) if message else message
        if code != EtatInscr.ERREUR:
            inscription.etat = 2
        inscription.save()
        if code != EtatInscr.OK:
            continue

        uid = prochain_uid_libre()
        if request.ecole == Ecole.enm:
            prenom = to_alphabetic(inscription.prenom)
            nom = to_alphabetic(inscription.nom)
            inscription.ident_cri = f"{prenom}.{nom}"
            inscription.mail_cri = f"{prenom}.{nom}@meteo.fr"
        creer_compte(inscription, uid, "ou=people," + request.ecole.dn)
        inscription.etat = 3
        inscription.save()

    c = {
        "inscriptions": inscriptions,
    }
    return render(request, "gestion/inscriptions/preparation.html", c)


@permission_needed("gerer_inscriptions")
def ajouter_effect(request):
    prec = datetime.now() - timedelta(minutes=5)
    Inscription.objects.filter(ecole=request.ecole, etat=0, date_ajout__lt=prec).update(etat=1)

    return redirect("gestion:inscriptions:preparation")


@permission_needed("gerer_inscriptions")
def recherche_ldap_cri(request):
    if request.method == "POST":
        c = {
            "recherche": True,
            "nom": request.POST.get("nom"),
            "prenom": request.POST.get("prenom"),
            "resultats": donnees_cri(request.POST.get("nom", ""), request.POST.get("prenom", ""))[::-1],
        }
    else:
        c = {"nom": "", "prenom": ""}

    return render(request, "gestion/inscriptions/recherche_ldap_cri.html", c)
